
% This code is part of the supplementary material to the ICCV 2011 paper
% "Multiclass Transfer Learning from Unconstrained Priors", L. Jie*, T.
% Tommasi*, B. Caputo (* equal authorship). 

% Copyright (C) 2010-2011, Tatiana Tommasi
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% Contact the authors: ttommasi [at] idiap.ch , jluo [at] idiap.ch


clear
addpath('classifiers')
addpath('obscure')

% [6,20,40] samples for each class and 3 permutations.
n_perm   = 3;
n_tr_set = [6 20 40];

n_te  = 50; 
n_cls = 10;
n_prior = 40;

acc_tl = zeros(numel(n_tr_set), n_perm);
acc_no = zeros(numel(n_tr_set), n_perm);
acc_pr = zeros(numel(n_tr_set), n_perm);

global Ktest Ytest Ptest
MEM = load('data/f_PHOG.mat');
YA = MEM.t;
MEM = load('data/p_source.mat');
PA = MEM.val;

rand('state', 3)
for nn = 1:numel(n_tr_set)

n_tr  = n_tr_set(nn);
K     = zeros(n_tr*n_cls, n_tr*n_cls);
Ktest = zeros(n_tr*n_cls, n_te*n_cls);

for pp = 1:n_perm
idx_tr = [];
idx_te = [];
for c=1:n_cls
  idx_x_cls = find(YA == c);
  idx_rand  = randperm(numel(idx_x_cls));
  idx_tr    = [ idx_tr; idx_x_cls(idx_rand(1:n_tr)) ];
  idx_te    = [ idx_te; idx_x_cls(idx_rand(n_tr+1:n_tr+n_te)) ];    
end

Y     = YA(idx_tr);
Ytest = YA(idx_te);
P     = PA(idx_tr, :)';
Ptest = PA(idx_te, :)';

hp.type  = 'rbf';
hp.gamma = 0.3174;
RAM   = load('data/f_PHOG');
XA    = RAM.p;    
K(:,:)     = compute_kernel(XA(idx_tr, :)', XA(idx_tr, :)', hp);
Ktest(:,:) = compute_kernel(XA(idx_tr, :)', XA(idx_te, :)', hp);

model_init.n_cla     = 10;
model_init.step      = 100*numel(Y);
model_init.T         = 300;
model_init.eachRound = @mktl_test;

model_init.C = 1;
model_init.p = 1.5; 

fprintf('\n\nTraining MKTL model ...\n');
model_tl = mktl_train(K, P, Y, model_init);
acc_tl(nn, pp) = model_tl.outputER(end).acc(2);

fprintf('\n\nTraining No-Transfer model ...\n');
model_no = svm_train_ova(Y, [ (1:numel(Y))' K ], ['-t 4 -c 1']);
[pred, acc, dec] = svm_predict_ova(Ytest, [(1:numel(Ytest))', Ktest'], model_no);
acc_no(nn, pp) = acc(1); 

fprintf('\n\nTraining Prior-Feature model ...\n');
model_pr = linear_train(Y, sparse(P'), ['-B 1 -s 4 -c 1']); 
[yhat, acc, dec] = linear_predict(Ytest, sparse(Ptest'), model_pr);
acc_pr(nn, pp) = acc(1);
end 
end 


% PLOT THE RESULT
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
X = [6 20 40];

acc_no_mean=mean(acc_no,2)*100;
std_no_mean=std(acc_no,0,2)*100;
acc_tl_mean=mean(acc_tl,2)*100;
std_tl_mean=std(acc_tl,0,2)*100;
acc_pr_mean=mean(acc_pr,2);
std_pr_mean=std(acc_pr,0,2);

errorbar(X, acc_no_mean, std_no_mean, '--r', 'LineWidth', 2); hold on
errorbar(X, acc_pr_mean, std_pr_mean, '--g', 'LineWidth', 2); hold on
errorbar(X, acc_tl_mean, std_tl_mean, 'k', 'LineWidth', 2); hold on

legend('No-Transfer', 'Prior-Features', 'MKTL',  'Location', 'Best');
xlabel('#. of training samples per class');
ylabel('Recognition Accuracy [%]');
